@@ -12,7 +12,18 @@ from unittest.mock import Mock
 import pytest
 
 import zigpy.application
-from zigpy.config import CONF_DATABASE, CONF_DEVICE, CONF_DEVICE_PATH
+from zigpy.config import (
+    CONF_DATABASE,
+    CONF_DEVICE,
+    CONF_DEVICE_PATH,
+    CONF_OTA,
+    CONF_OTA_IKEA,
+    CONF_OTA_INOVELLI,
+    CONF_OTA_LEDVANCE,
+    CONF_OTA_SALUS,
+    CONF_OTA_SONOFF,
+    CONF_OTA_THIRDREALITY,
+)
 import zigpy.state as app_state
 import zigpy.types as t
 import zigpy.zdo.types as zdo_t
@@ -65,6 +76,8 @@ class App(zigpy.application.ControllerApplication):
         dev = add_initialized_device(
             app=self, nwk=self.state.node_info.nwk, ieee=self.state.node_info.ieee
         )
+        dev.model = "Coordinator Model"
+        dev.manufacturer = "Coordinator Manufacturer"
 
         dev.zdo.Mgmt_NWK_Update_req = AsyncMock(
             return_value=[
@@ -85,7 +98,7 @@ class App(zigpy.application.ControllerApplication):
     async def permit_ncp(self, time_s=60):
         pass
 
-    async def permit_with_key(self, node, code, time_s=60):
+    async def permit_with_link_key(self, node, link_key, time_s=60):
         pass
 
     async def reset_network_info(self):
@@ -120,6 +133,14 @@ def make_app(
         {
             CONF_DATABASE: None,
             CONF_DEVICE: {CONF_DEVICE_PATH: "/dev/null"},
+            CONF_OTA: {
+                CONF_OTA_IKEA: False,
+                CONF_OTA_INOVELLI: False,
+                CONF_OTA_LEDVANCE: False,
+                CONF_OTA_SALUS: False,
+                CONF_OTA_SONOFF: False,
+                CONF_OTA_THIRDREALITY: False,
+            },
         },
         config_updates,
     )
