@@ -25,7 +25,7 @@ import zigpy.zcl
 from zigpy.zcl.foundation import Status as ZCLStatus
 from zigpy.zdo import types as zdo_t
 
-from tests.async_mock import AsyncMock, MagicMock, patch
+from tests.async_mock import AsyncMock, MagicMock, call, patch
 from tests.conftest import make_app, make_ieee
 from tests.test_backups import backup_factory  # noqa: F401
 
@@ -456,6 +456,56 @@ async def test_attribute_update(tmp_path, dev_init):
     await app2.shutdown()
 
 
+@patch.object(Device, "schedule_initialize", new=mock_dev_init(True))
+async def test_attribute_update_short_interval(tmp_path):
+    """Test updating an attribute twice in a short interval."""
+
+    db = tmp_path / "test.db"
+    app = await make_app_with_db(db)
+
+    ieee = make_ieee()
+    app.handle_join(99, ieee, 0)
+
+    dev = app.get_device(ieee)
+    ep = dev.add_endpoint(3)
+    ep.status = zigpy.endpoint.Status.ZDO_INIT
+    ep.profile_id = 260
+    ep.device_type = profiles.zha.DeviceType.PUMP
+    clus = ep.add_input_cluster(0x0000)
+    ep.add_output_cluster(0x0001)
+    clus.update_attribute(0x0004, "Custom")
+    clus.update_attribute(0x0005, "Model")
+    app.device_initialized(dev)
+
+    # wait for the device initialization to write attribute cache to db
+    await asyncio.sleep(0.01)
+
+    # update an attribute twice in a short interval
+    clus.update_attribute(0x4000, "1.0")
+    attr_update_time_first = clus._attr_last_updated[0x4000]
+
+    # update attribute again 10 seconds later
+    fake_time = datetime.utcnow() + timedelta(seconds=10)
+    with freezegun.freeze_time(fake_time):
+        clus.update_attribute(0x4000, "2.0")
+
+    await app.shutdown()
+
+    # Everything should've been saved - check that it re-loads
+    app2 = await make_app_with_db(db)
+    dev = app2.get_device(ieee)
+
+    clus = dev.endpoints[3].in_clusters[0x0000]
+    assert clus._attr_cache[0x4000] == "2.0"  # verify second attribute update was saved
+
+    # verify the first update attribute time was not overwritten, as it was within the short interval
+    assert (attr_update_time_first - clus._attr_last_updated[0x0004]) < timedelta(
+        seconds=0.1
+    )
+
+    await app2.shutdown()
+
+
 @patch("zigpy.topology.REQUEST_DELAY", (0, 0))
 @patch.object(Device, "schedule_initialize", new=mock_dev_init(True))
 async def test_topology(tmp_path):
@@ -1023,3 +1073,48 @@ async def test_appdb_network_backups(tmp_path, backup_factory):  # noqa: F811
     assert app3.backups.backups[0] == new_backup
     assert app3.backups.backups[0] != backup
     await app3.shutdown()
+
+
+async def test_appdb_network_backups_format_change(
+    tmp_path, backup_factory
+):  # noqa: F811
+    db = tmp_path / "test.db"
+
+    backup = backup_factory()
+    backup.as_dict = MagicMock(return_value={"some new key": 1, **backup.as_dict()})
+
+    app1 = await make_app_with_db(db)
+    app1.backups.add_backup(backup)
+    await app1.shutdown()
+
+    # The backup is reloaded from the database as well
+    app2 = await make_app_with_db(db)
+    assert len(app2.backups.backups) == 1
+    assert app2.backups.backups[0] == backup
+
+    new_backup = backup_factory()
+    new_backup.network_info.network_key.tx_counter += 10000
+
+    app2.backups.add_backup(new_backup)
+    await app2.shutdown()
+
+    # The database will contain only the single backup
+    with patch("zigpy.backups.BackupManager.add_backup") as mock_add_backup:
+        app3 = await make_app_with_db(db)
+        await app3.shutdown()
+
+    assert mock_add_backup.mock_calls == [call(new_backup, suppress_event=True)]
+
+
+async def test_appdb_persist_coordinator_info(tmp_path):  # noqa: F811
+    db = tmp_path / "test.db"
+
+    with patch(
+        "zigpy.appdb.PersistingListener._save_attribute_cache",
+        wraps=zigpy.appdb.PersistingListener._save_attribute_cache,
+    ) as mock_save_attr_cache:
+        app = await make_app_with_db(db)
+        await app.initialize()
+        await app.shutdown()
+
+    assert mock_save_attr_cache.mock_calls == [call(app._device.endpoints[1])]
