@@ -5,9 +5,9 @@ from unittest import mock
 import pytest
 
 from zigpy import listeners
-import zigpy.device
 from zigpy.zcl import foundation
 import zigpy.zcl.clusters.general
+import zigpy.zdo.types as zdo_t
 
 
 def make_hdr(cmd, **kwargs):
@@ -24,7 +24,6 @@ toggle = zigpy.zcl.clusters.general.OnOff.commands_by_name["toggle"].schema
 
 async def test_future_listener():
     listener = listeners.FutureListener(
-        device=mock.Mock(spec_set=zigpy.device.Device),
         matchers=[
             query_next_image(manufacturer_code=0x1234),
             on(),
@@ -72,7 +71,6 @@ async def test_future_listener():
 
 async def test_future_listener_cancellation():
     listener = listeners.FutureListener(
-        device=mock.Mock(spec_set=zigpy.device.Device),
         matchers=[],
         future=asyncio.get_running_loop().create_future(),
     )
@@ -87,7 +85,6 @@ async def test_future_listener_cancellation():
 
 async def test_callback_listener():
     listener = listeners.CallbackListener(
-        device=mock.Mock(spec_set=zigpy.device.Device),
         matchers=[
             query_next_image(manufacturer_code=0x1234),
             on(),
@@ -119,27 +116,8 @@ async def test_callback_listener():
     ]
 
 
-async def test_callback_listener_async():
-    listener = listeners.CallbackListener(
-        device=mock.Mock(spec_set=zigpy.device.Device),
-        matchers=[
-            on(),
-        ],
-        callback=mock.AsyncMock(),
-    )
-
-    assert not listener.resolve(make_hdr(off()), off())
-    assert listener.resolve(make_hdr(on()), on())
-
-    await asyncio.sleep(0.1)
-
-    assert listener.callback.mock_calls == [mock.call(make_hdr(on()), on())]
-    assert listener.device.application.create_task.call_count == 1
-
-
 async def test_callback_listener_error(caplog):
     listener = listeners.CallbackListener(
-        device=mock.Mock(spec_set=zigpy.device.Device),
         matchers=[
             on(),
         ],
@@ -151,3 +129,66 @@ async def test_callback_listener_error(caplog):
 
     assert "Caught an exception while executing callback" in caplog.text
     assert "RuntimeError: Uh oh" in caplog.text
+
+
+async def test_listener_callback_matches():
+    listener = listeners.CallbackListener(
+        matchers=[lambda hdr, command: True],
+        callback=mock.Mock(),
+    )
+
+    assert listener.resolve(make_hdr(off()), off())
+    assert listener.callback.mock_calls == [mock.call(make_hdr(off()), off())]
+
+
+async def test_listener_callback_no_matches():
+    listener = listeners.CallbackListener(
+        matchers=[lambda hdr, command: False],
+        callback=mock.Mock(),
+    )
+
+    assert not listener.resolve(make_hdr(off()), off())
+    assert listener.callback.mock_calls == []
+
+
+async def test_listener_callback_invalid_matcher(caplog):
+    listener = listeners.CallbackListener(
+        matchers=[object()],
+        callback=mock.Mock(),
+    )
+
+    with caplog.at_level(logging.WARNING):
+        assert not listener.resolve(make_hdr(off()), off())
+
+    assert listener.callback.mock_calls == []
+    assert f"Matcher {listener.matchers[0]!r} and command" in caplog.text
+
+
+async def test_listener_callback_invalid_call(caplog):
+    listener = listeners.CallbackListener(
+        matchers=[on()],
+        callback=mock.Mock(),
+    )
+
+    with caplog.at_level(logging.WARNING):
+        assert not listener.resolve(make_hdr(on()), b"data")
+
+    assert listener.callback.mock_calls == []
+    assert f"Matcher {listener.matchers[0]!r} and command" in caplog.text
+
+
+async def test_listener_callback_zdo(caplog):
+    listener = listeners.CallbackListener(
+        matchers=[
+            query_next_image(manufacturer_code=0x1234),
+        ],
+        callback=mock.Mock(),
+    )
+
+    zdo_hdr = zdo_t.ZDOHeader(command_id=zdo_t.ZDOCmd.NWK_addr_req, tsn=0x01)
+    zdo_cmd = [0x0000]
+
+    with caplog.at_level(logging.WARNING):
+        assert not listener.resolve(zdo_hdr, zdo_cmd)
+
+    assert caplog.text == ""
