@@ -196,7 +196,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     def enqueue(self, cb_name: str, *args) -> None:
         """Enqueue an async callback handler action."""
         if not self.running:
-            LOGGER.warning("Discarding %s event", cb_name)
+            LOGGER.debug("Discarding %s event", cb_name)
             return
         self._callback_handlers.put_nowait((cb_name, args))
 
@@ -556,7 +556,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 SET value=excluded.value, last_updated=excluded.last_updated
                 WHERE
                     value != excluded.value
-                    AND :timestamp - last_updated > :min_update_delta
+                    OR :timestamp - last_updated > :min_update_delta
             """
         await self.execute(
             q,
@@ -585,12 +585,13 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         await self._db.commit()
 
     def network_backup_removed(self, backup: zigpy.backups.NetworkBackup) -> None:
-        self.enqueue("_network_backup_removed", json.dumps(backup.as_dict()))
+        self.enqueue("_network_backup_removed", backup.backup_time)
 
-    async def _network_backup_removed(self, backup_json: str) -> None:
-        q = f"""DELETE FROM network_backups{DB_V} WHERE backup_json=?"""
+    async def _network_backup_removed(self, backup_time: datetime) -> None:
+        q = f"""DELETE FROM network_backups{DB_V}
+                    WHERE json_extract(backup_json, '$.backup_time')=?"""
 
-        await self.execute(q, (backup_json,))
+        await self.execute(q, (backup_time.isoformat(),))
         await self._db.commit()
 
     async def load(self) -> None:
