@@ -80,3 +80,15 @@ def cv_deprecated(message: str) -> typing.Callable[[typing.Any], typing.Any]:
         return obj
 
     return wrapper
+
+
+def cv_exact_object(expected_value: str) -> typing.Callable[[typing.Any], bool]:
+    """Factory function for creating an exact object comparison validator."""
+
+    def wrapper(obj: typing.Any) -> typing.Any:
+        if obj != expected_value:
+            return False
+
+        return expected_value
+
+    return wrapper
