@@ -1,10 +1,19 @@
 from __future__ import annotations
 
+import typing
+
+if typing.TYPE_CHECKING:
+    import zigpy.backups
+
 
 class ZigbeeException(Exception):
     """Base exception class"""
 
 
+class ParsingError(ZigbeeException):
+    """Failed to parse a frame"""
+
+
 class ControllerException(ZigbeeException):
     """Application controller failed in some way."""
 
@@ -21,6 +30,10 @@ class DeliveryError(ZigbeeException):
         self.status = status
 
 
+class SendError(DeliveryError):
+    """Message could not be enqueued."""
+
+
 class InvalidResponse(ZigbeeException):
     """A ZDO or ZCL response has an unsuccessful status code"""
 
@@ -44,6 +57,16 @@ class FormationFailure(RadioException):
 class NetworkSettingsInconsistent(ZigbeeException):
     """Loaded network settings are different from what is in the database"""
 
+    def __init__(
+        self,
+        message: str,
+        new_state: zigpy.backups.NetworkBackup,
+        old_state: zigpy.backups.NetworkBackup,
+    ) -> None:
+        super().__init__(message)
+        self.new_state = new_state
+        self.old_state = old_state
+
 
 class CorruptDatabase(ZigbeeException):
     """The SQLite database is corrupt or otherwise inconsistent"""
