@@ -2,21 +2,22 @@ from __future__ import annotations
 
 import asyncio
 import dataclasses
+import inspect
 import logging
 import typing
 
+from zigpy.util import Singleton
 from zigpy.zcl import foundation
 import zigpy.zdo.types as zdo_t
 
-if typing.TYPE_CHECKING:
-    import zigpy.device
-
 LOGGER = logging.getLogger(__name__)
 
 
+ANY_DEVICE = Singleton("ANY_DEVICE")
+
+
 @dataclasses.dataclass(frozen=True)
 class BaseRequestListener:
-    device: zigpy.device.Device
     matchers: tuple[MatcherType]
 
     def resolve(
@@ -29,12 +30,23 @@ class BaseRequestListener:
         """
 
         for matcher in self.matchers:
-            if isinstance(matcher, foundation.CommandSchema) and isinstance(
-                command, foundation.CommandSchema
-            ):
+            match = None
+            is_matcher_cmd = isinstance(matcher, foundation.CommandSchema)
+
+            if is_matcher_cmd and isinstance(command, foundation.CommandSchema):
                 match = command.matches(matcher)
-            else:
+            elif is_matcher_cmd and isinstance(hdr, zdo_t.ZDOHeader):
+                # FIXME: ZDO does not use command schemas and cannot be matched
+                pass
+            elif callable(matcher):
                 match = matcher(hdr, command)
+            else:
+                LOGGER.warning(
+                    "Matcher %r and command %r %r are incompatible",
+                    matcher,
+                    hdr,
+                    command,
+                )
 
             if match:
                 return self._resolve(hdr, command)
@@ -88,6 +100,7 @@ class CallbackListener(BaseRequestListener):
     callback: typing.Callable[
         [foundation.ZCLHeader | zdo_t.ZDOHeader, foundation.CommandSchema], typing.Any
     ]
+    _tasks: set[asyncio.Task] = dataclasses.field(default_factory=set)
 
     def _resolve(
         self,
@@ -95,11 +108,13 @@ class CallbackListener(BaseRequestListener):
         command: foundation.CommandSchema,
     ) -> bool:
         try:
-            result = self.callback(hdr, command)
-
-            # Run coroutines in the background
-            if asyncio.iscoroutine(result):
-                self.device.application.create_task(result)
+            potential_awaitable = self.callback(hdr, command)
+            if inspect.isawaitable(potential_awaitable):
+                task: asyncio.Task = asyncio.get_running_loop().create_task(
+                    potential_awaitable, name="CallbackListener"
+                )
+                self._tasks.add(task)
+                task.add_done_callback(self._tasks.remove)
         except Exception:
             LOGGER.warning(
                 "Caught an exception while executing callback", exc_info=True
