@@ -37,13 +37,13 @@ class HeaderString(str):
     _size = 32
 
     @classmethod
-    def deserialize(cls, data):
+    def deserialize(cls, data: bytes) -> tuple[HeaderString, bytes]:
         if len(data) < cls._size:
             raise ValueError(f"Data is too short. Should be at least {cls._size}")
         raw = data[: cls._size].split(b"\x00")[0]
         return cls(raw.decode("utf8", errors="replace")), data[cls._size :]
 
-    def serialize(self):
+    def serialize(self) -> bytes:
         return self.encode("utf8").ljust(self._size, b"\x00")
 
 
@@ -102,11 +102,11 @@ class OTAImageHeader(t.Struct):
         return bool(self.field_control & FieldControl.HARDWARE_VERSIONS_PRESENT)
 
     @property
-    def key(self):
+    def key(self) -> ImageKey:
         return ImageKey(self.manufacturer_id, self.image_type)
 
     @classmethod
-    def deserialize(cls, data) -> tuple[OTAImageHeader, bytes]:
+    def deserialize(cls, data: bytes) -> tuple[OTAImageHeader, bytes]:
         hdr, data = super().deserialize(data)
         if hdr.upgrade_file_id != cls.MAGIC_VALUE:
             raise ValueError(
@@ -136,8 +136,7 @@ class SubElement(t.Struct):
 
 
 class BaseOTAImage:
-    """
-    Base OTA image container type. Not all images are valid Zigbee OTA images but are
+    """Base OTA image container type. Not all images are valid Zigbee OTA images but are
     nonetheless accepted by devices. Only requirement is that the image contains a valid
     OTAImageHeader property and can be serialized/deserialized.
     """
@@ -153,15 +152,13 @@ class BaseOTAImage:
 
 
 class OTAImage(t.Struct, BaseOTAImage):
-    """
-    Zigbee OTA image according to 11.4 of the ZCL specification.
-    """
+    """Zigbee OTA image according to 11.4 of the ZCL specification."""
 
     header: OTAImageHeader
     subelements: t.List[SubElement]
 
     @classmethod
-    def deserialize(cls, data) -> tuple[OTAImage, bytes]:
+    def deserialize(cls, data: bytes) -> tuple[OTAImage, bytes]:
         hdr, data = OTAImageHeader.deserialize(data)
         elements_len = hdr.image_size - hdr.header_length
 
@@ -177,7 +174,7 @@ class OTAImage(t.Struct, BaseOTAImage):
 
         return image, data
 
-    def serialize(self):
+    def serialize(self) -> bytes:
         res = super().serialize()
         assert len(res) == self.header.image_size
 
@@ -186,8 +183,7 @@ class OTAImage(t.Struct, BaseOTAImage):
 
 @attr.s
 class HueSBLOTAImage(BaseOTAImage):
-    """
-    Unique OTA image format for certain Hue devices. Starts with a valid header but does
+    """Unique OTA image format for certain Hue devices. Starts with a valid header but does
     not contain any valid subelements beyond that point.
     """
 
@@ -200,7 +196,7 @@ class HueSBLOTAImage(BaseOTAImage):
         return self.header.serialize() + self.data
 
     @classmethod
-    def deserialize(cls, data) -> tuple[HueSBLOTAImage, bytes]:
+    def deserialize(cls, data: bytes) -> tuple[HueSBLOTAImage, bytes]:
         header, remaining_data = OTAImageHeader.deserialize(data)
         firmware = remaining_data[: header.image_size - len(header.serialize())]
 
@@ -223,9 +219,7 @@ class HueSBLOTAImage(BaseOTAImage):
 
 
 def parse_ota_image(data: bytes) -> tuple[BaseOTAImage, bytes]:
-    """
-    Attempts to extract any known OTA image type from data. Does not validate firmware.
-    """
+    """Attempts to extract any known OTA image type from data. Does not validate firmware."""
 
     if len(data) > 4 and int.from_bytes(data[0:4], "little") + 21 == len(data):
         # Legrand OTA images are prefixed with their unwrapped size and include a 1 + 16
