@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import logging
-from typing import Any, Callable, Coroutine, Iterable
+import typing
 
 from zigpy.const import (  # noqa: F401
     SIG_ENDPOINTS,
@@ -19,16 +19,22 @@ import zigpy.device
 import zigpy.endpoint
 from zigpy.quirks.registry import DeviceRegistry  # noqa: F401
 import zigpy.types as t
+from zigpy.types.basic import uint16_t
 import zigpy.zcl
 import zigpy.zcl.foundation as foundation
 
+if typing.TYPE_CHECKING:
+    from zigpy.application import ControllerApplication
+
 _LOGGER = logging.getLogger(__name__)
 
 _DEVICE_REGISTRY = DeviceRegistry()
 _uninitialized_device_message_handlers = []
 
 
-def get_device(device: zigpy.device.Device, registry: DeviceRegistry | None = None):
+def get_device(
+    device: zigpy.device.Device, registry: DeviceRegistry | None = None
+) -> zigpy.device.Device:
     """Get a CustomDevice object, if one is available"""
     if registry is None:
         return _DEVICE_REGISTRY.get_device(device)
@@ -46,7 +52,7 @@ def get_quirk_list(
     return registry.registry[manufacturer][model]
 
 
-def register_uninitialized_device_message_handler(handler: Callable) -> None:
+def register_uninitialized_device_message_handler(handler: typing.Callable) -> None:
     """Register an handler for messages received by uninitialized devices.
 
     each handler is passed same parameters as
@@ -57,14 +63,20 @@ def register_uninitialized_device_message_handler(handler: Callable) -> None:
 
 
 class CustomDevice(zigpy.device.Device):
-    replacement: dict[str, Any] = {}
+    replacement: dict[str, typing.Any] = {}
     signature = None
 
-    def __init_subclass__(cls):
+    def __init_subclass__(cls) -> None:
         if getattr(cls, "signature", None) is not None:
             _DEVICE_REGISTRY.add_to_registry(cls)
 
-    def __init__(self, application, ieee, nwk, replaces):
+    def __init__(
+        self,
+        application: ControllerApplication,
+        ieee: t.EUI64,
+        nwk: t.NWK,
+        replaces: zigpy.device.Device,
+    ) -> None:
         super().__init__(application, ieee, nwk)
 
         def set_device_attr(attr):
@@ -81,10 +93,12 @@ class CustomDevice(zigpy.device.Device):
         set_device_attr(SIG_MANUFACTURER)
         set_device_attr(SIG_MODEL)
         set_device_attr(SIG_SKIP_CONFIG)
-        for endpoint_id, endpoint in self.replacement.get(SIG_ENDPOINTS, {}).items():
+        for endpoint_id, _endpoint in self.replacement.get(SIG_ENDPOINTS, {}).items():
             self.add_endpoint(endpoint_id, replace_device=replaces)
 
-    def add_endpoint(self, endpoint_id, replace_device=None) -> zigpy.endpoint.Endpoint:
+    def add_endpoint(
+        self, endpoint_id: int, replace_device: zigpy.device.Device | None = None
+    ) -> zigpy.endpoint.Endpoint:
         if endpoint_id not in self.replacement.get(SIG_ENDPOINTS, {}):
             return super().add_endpoint(endpoint_id)
 
@@ -103,7 +117,13 @@ class CustomDevice(zigpy.device.Device):
 
 
 class CustomEndpoint(zigpy.endpoint.Endpoint):
-    def __init__(self, device, endpoint_id, replacement_data, replace_device):
+    def __init__(
+        self,
+        device: CustomDevice,
+        endpoint_id: int,
+        replacement_data: dict[str, typing.Any],
+        replace_device: zigpy.device.Device,
+    ) -> None:
         super().__init__(device, endpoint_id)
 
         def set_device_attr(attr):
@@ -137,7 +157,7 @@ class CustomEndpoint(zigpy.endpoint.Endpoint):
 
 class CustomCluster(zigpy.zcl.Cluster):
     _skip_registry = True
-    _CONSTANT_ATTRIBUTES: dict[int, Any] | None = None
+    _CONSTANT_ATTRIBUTES: dict[int, typing.Any] | None = None
 
     manufacturer_id_override: t.uint16_t | None = None
 
@@ -146,7 +166,7 @@ class CustomCluster(zigpy.zcl.Cluster):
         """Return True if cluster_id is within manufacturer specific range."""
         return 0xFC00 <= self.cluster_id <= 0xFFFF
 
-    def _has_manuf_attr(self, attrs_to_process: Iterable | list | dict) -> bool:
+    def _has_manuf_attr(self, attrs_to_process: typing.Iterable | list | dict) -> bool:
         """Return True if contains a manufacturer specific attribute."""
         if self._is_manuf_specific:
             return True
@@ -166,10 +186,9 @@ class CustomCluster(zigpy.zcl.Cluster):
         *args,
         manufacturer: int | t.uint16_t | None = None,
         expect_reply: bool = True,
-        tries: int = 1,
         tsn: int | t.uint8_t | None = None,
-        **kwargs: Any,
-    ) -> Coroutine:
+        **kwargs: typing.Any,
+    ) -> typing.Coroutine:
         command = self.server_commands[command_id]
 
         if manufacturer is None and (
@@ -184,7 +203,6 @@ class CustomCluster(zigpy.zcl.Cluster):
             *args,
             manufacturer=manufacturer,
             expect_reply=expect_reply,
-            tries=tries,
             tsn=tsn,
             **kwargs,
         )
@@ -195,8 +213,8 @@ class CustomCluster(zigpy.zcl.Cluster):
         *args,
         manufacturer: int | t.uint16_t | None = None,
         tsn: int | t.uint8_t | None = None,
-        **kwargs: Any,
-    ) -> Coroutine:
+        **kwargs: typing.Any,
+    ):
         command = self.client_commands[command_id]
 
         if manufacturer is None and (
@@ -214,7 +232,9 @@ class CustomCluster(zigpy.zcl.Cluster):
             **kwargs,
         )
 
-    async def read_attributes_raw(self, attributes, manufacturer=None):
+    async def read_attributes_raw(
+        self, attributes: list[uint16_t], manufacturer: uint16_t | None = None
+    ):
         if not self._CONSTANT_ATTRIBUTES:
             return await super().read_attributes_raw(
                 attributes, manufacturer=manufacturer
@@ -318,6 +338,24 @@ class CustomCluster(zigpy.zcl.Cluster):
             attributes, *args, manufacturer=manufacturer, **kwargs
         )
 
+    def get(self, key: int | str, default: typing.Any | None = None) -> typing.Any:
+        """Get cached attribute."""
+
+        try:
+            attr_def = self.find_attribute(key)
+        except KeyError:
+            return super().get(key, default)
+
+        # Ensure we check the constant attributes dictionary first, since their values
+        # will not be in the attribute cache but can be read immediately.
+        if (
+            self._CONSTANT_ATTRIBUTES is not None
+            and attr_def.id in self._CONSTANT_ATTRIBUTES
+        ):
+            return self._CONSTANT_ATTRIBUTES[attr_def.id]
+
+        return super().get(key, default)
+
 
 def handle_message_from_uninitialized_sender(
     sender: zigpy.device.Device,
