@@ -1,7 +1,9 @@
+from __future__ import annotations
+
 import collections
 import itertools
 import logging
-from typing import Dict, List, Optional, Union
+import typing
 
 from zigpy.const import (
     SIG_ENDPOINTS,
@@ -18,12 +20,14 @@ from zigpy.typing import CustomDeviceType, DeviceType
 
 _LOGGER = logging.getLogger(__name__)
 
-TYPE_MODEL_QUIRKS_LIST = Dict[Optional[str], List["zigpy.quirks.CustomDevice"]]
-TYPE_MANUF_QUIRKS_DICT = Dict[Optional[str], TYPE_MODEL_QUIRKS_LIST]
+TYPE_MANUF_QUIRKS_DICT = typing.Dict[
+    typing.Optional[str],
+    typing.Dict[typing.Optional[str], typing.List["zigpy.quirks.CustomDevice"]],
+]
 
 
 class DeviceRegistry:
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         self._registry: TYPE_MANUF_QUIRKS_DICT = collections.defaultdict(
             lambda: collections.defaultdict(list)
         )
@@ -51,7 +55,7 @@ class DeviceRegistry:
             model = custom_device.signature.get(SIG_MODEL)
             self.registry[manufacturer][model].remove(custom_device)
 
-    def get_device(self, device: DeviceType) -> Union[CustomDeviceType, DeviceType]:
+    def get_device(self, device: DeviceType) -> CustomDeviceType | DeviceType:
         """Get a CustomDevice object, if one is available"""
         if isinstance(device, zigpy.quirks.CustomDevice):
             return device
@@ -70,13 +74,12 @@ class DeviceRegistry:
         ):
             _LOGGER.debug("Considering %s", candidate)
 
-            if not device.model == candidate.signature.get(SIG_MODEL, device.model):
+            if device.model != candidate.signature.get(SIG_MODEL, device.model):
                 _LOGGER.debug("Fail, because device model mismatch: '%s'", device.model)
                 continue
 
-            if not (
-                device.manufacturer
-                == candidate.signature.get(SIG_MANUFACTURER, device.manufacturer)
+            if device.manufacturer != candidate.signature.get(
+                SIG_MANUFACTURER, device.manufacturer
             ):
                 _LOGGER.debug(
                     "Fail, because device manufacturer mismatch: '%s'",
@@ -97,11 +100,9 @@ class DeviceRegistry:
                 continue
 
             if not all(
-                [
-                    device[eid].profile_id
-                    == sig[eid].get(SIG_EP_PROFILE, device[eid].profile_id)
-                    for eid in sig
-                ]
+                device[eid].profile_id
+                == sig[eid].get(SIG_EP_PROFILE, device[eid].profile_id)
+                for eid in sig
             ):
                 _LOGGER.debug(
                     "Fail because profile_id mismatch on at least one endpoint"
@@ -109,11 +110,9 @@ class DeviceRegistry:
                 continue
 
             if not all(
-                [
-                    device[eid].device_type
-                    == sig[eid].get(SIG_EP_TYPE, device[eid].device_type)
-                    for eid in sig
-                ]
+                device[eid].device_type
+                == sig[eid].get(SIG_EP_TYPE, device[eid].device_type)
+                for eid in sig
             ):
                 _LOGGER.debug(
                     "Fail because device_type mismatch on at least one endpoint"
@@ -121,10 +120,8 @@ class DeviceRegistry:
                 continue
 
             if not all(
-                [
-                    self._match(device[eid].in_clusters, ep.get(SIG_EP_INPUT, []))
-                    for eid, ep in sig.items()
-                ]
+                self._match(device[eid].in_clusters, ep.get(SIG_EP_INPUT, []))
+                for eid, ep in sig.items()
             ):
                 _LOGGER.debug(
                     "Fail because input cluster mismatch on at least one endpoint"
@@ -132,10 +129,8 @@ class DeviceRegistry:
                 continue
 
             if not all(
-                [
-                    self._match(device[eid].out_clusters, ep.get(SIG_EP_OUTPUT, []))
-                    for eid, ep in sig.items()
-                ]
+                self._match(device[eid].out_clusters, ep.get(SIG_EP_OUTPUT, []))
+                for eid, ep in sig.items()
             ):
                 _LOGGER.debug(
                     "Fail because output cluster mismatch on at least one endpoint"
@@ -151,11 +146,11 @@ class DeviceRegistry:
         return device
 
     @staticmethod
-    def _match(a, b):
+    def _match(a: dict | typing.Iterable, b: dict | typing.Iterable) -> bool:
         return set(a) == set(b)
 
     @property
-    def registry(self):
+    def registry(self) -> TYPE_MANUF_QUIRKS_DICT:
         return self._registry
 
     def __contains__(self, device: CustomDeviceType) -> bool:
