@@ -1,16 +1,24 @@
 from __future__ import annotations
 
 import asyncio
+import logging
 import typing
 import urllib.parse
 
 import async_timeout
 import serial as pyserial
-import serial_asyncio as pyserial_asyncio
 
+LOGGER = logging.getLogger(__name__)
 DEFAULT_SOCKET_PORT = 6638
 SOCKET_CONNECT_TIMEOUT = 5
 
+try:
+    import serial_asyncio_fast as pyserial_asyncio
+
+    LOGGER.info("Using pyserial-asyncio-fast in place of pyserial-asyncio")
+except ImportError:
+    import serial_asyncio as pyserial_asyncio
+
 
 async def create_serial_connection(
     loop: asyncio.BaseEventLoop,
@@ -24,6 +32,9 @@ async def create_serial_connection(
     """Wrapper around pyserial-asyncio that transparently substitutes a normal TCP
     transport and protocol when a `socket` connection URI is provided.
     """
+    baudrate: int | None = kwargs.get("baudrate")
+    LOGGER.debug("Opening a serial connection to %r (%s baudrate)", url, baudrate)
+
     parsed_url = urllib.parse.urlparse(url)
 
     if parsed_url.scheme in ("socket", "tcp"):
