@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import dataclasses
+from datetime import datetime, timezone
 import enum
 import typing
 
@@ -566,6 +567,10 @@ class ZigbeePacket(BaseDataclassMixin):
     it to zigpy for every type of packet.
     """
 
+    timestamp: datetime = dataclasses.field(
+        compare=False, default_factory=lambda: datetime.now(timezone.utc)
+    )
+
     # Set to `None` when the packet is outgoing
     src: AddrModeAddress | None = dataclasses.field(default=None)
     src_ep: basic.uint8_t | None = dataclasses.field(default=None)
