@@ -125,6 +125,8 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
                     type=attr_type,
                     is_manufacturer_specific=attr_manuf_specific,
                 )
+            else:
+                attr = attr.replace(id=attr_id)
 
             cls.attributes[attr.id] = attr.replace(id=attr_id)
 
@@ -264,20 +266,20 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
 
         if hdr.frame_control.frame_type == foundation.FrameType.CLUSTER_COMMAND:
             # Cluster command
-            if hdr.direction == foundation.Direction.Client_to_Server:
+            if hdr.direction == foundation.Direction.Server_to_Client:
                 commands = self.client_commands
             else:
                 commands = self.server_commands
 
             if hdr.command_id not in commands:
-                self.warning("Unknown cluster command %s %s", hdr.command_id, data)
+                self.debug("Unknown cluster command %s %s", hdr.command_id, data)
                 return hdr, data
 
             command = commands[hdr.command_id]
         else:
             # General command
             if hdr.command_id not in foundation.GENERAL_COMMANDS:
-                self.warning("Unknown foundation command %s %s", hdr.command_id, data)
+                self.debug("Unknown foundation command %s %s", hdr.command_id, data)
                 return hdr, data
 
             command = foundation.GENERAL_COMMANDS[hdr.command_id]
@@ -288,7 +290,7 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
         self.debug("Decoded ZCL frame: %s:%r", type(self).__name__, response)
 
         if data:
-            self.warning("Data remains after deserializing ZCL frame: %r", data)
+            self.debug("Data remains after deserializing ZCL frame: %r", data)
 
         return hdr, response
 
@@ -311,14 +313,14 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
             schema = convert_list_schema(
                 command_id=command_id,
                 schema=schema,
-                direction=foundation.Direction.Server_to_Client,
+                direction=foundation.Direction.Client_to_Server,
             )
 
         request = schema(*args, **kwargs)  # type:ignore[operator]
         request.serialize()  # Throw an error before generating a new TSN
 
         if tsn is None:
-            tsn = self._endpoint.device.application.get_sequence()
+            tsn = self._endpoint.device.get_sequence()
 
         frame_control = foundation.FrameControl(
             frame_type=(
@@ -360,9 +362,9 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
             tsn=tsn,
             disable_default_response=self.is_client,
             direction=(
-                foundation.Direction.Client_to_Server
+                foundation.Direction.Server_to_Client
                 if self.is_client
-                else foundation.Direction.Server_to_Client
+                else foundation.Direction.Client_to_Server
             ),
             args=args,
             kwargs=kwargs,
@@ -397,7 +399,11 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
             manufacturer=manufacturer,
             tsn=tsn,
             disable_default_response=True,
-            direction=foundation.Direction.Client_to_Server,
+            direction=(
+                foundation.Direction.Server_to_Client
+                if self.is_client
+                else foundation.Direction.Client_to_Server
+            ),
             args=args,
             kwargs=kwargs,
         )
@@ -868,7 +874,7 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
     ):
         command = foundation.GENERAL_COMMANDS[command_id]
 
-        if command.direction == foundation.Direction.Client_to_Server:
+        if command.direction == foundation.Direction.Server_to_Client:
             # should reply be retryable?
             return self.reply(
                 True,
